#region Copyright notice and license

// Copyright 2015 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#endregion

using System;
using System.Threading;

namespace Grpc.Core.Internal
{
    internal class DebugStats
    {
        public readonly AtomicCounter PendingBatchCompletions = new AtomicCounter();

        /// <summary>
        /// Checks the debug stats and take action for any inconsistency found.
        /// </summary>
        public void CheckOK()
        {
            var pendingBatchCompletions = PendingBatchCompletions.Count;
            if (pendingBatchCompletions != 0)
            {
                DebugWarning(string.Format("Detected {0} pending batch completions.", pendingBatchCompletions));
            }
        }

        private void DebugWarning(string message)
        {
            throw new Exception("Shutdown check: " + message);
        }
    }
}
