// Copyright 2019 The gRPC Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
syntax = "proto3";

message BenchmarkRequest {
  int32 message_size = 1;
  int32 response_count = 2;
}

message BenchmarkResponse {
  bytes response = 1;
}

service UnaryStreamBenchmarkService {
  rpc Benchmark(BenchmarkRequest) returns (stream BenchmarkResponse);
}
