/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/api/v2/listener/listener_components.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_API_V2_LISTENER_LISTENER_COMPONENTS_PROTO_UPB_H_
#define ENVOY_API_V2_LISTENER_LISTENER_COMPONENTS_PROTO_UPB_H_

#include "upb/generated_util.h"
#include "upb/msg.h"
#include "upb/decode.h"
#include "upb/encode.h"

#include "upb/port_def.inc"

#ifdef __cplusplus
extern "C" {
#endif

struct envoy_api_v2_listener_Filter;
struct envoy_api_v2_listener_FilterChainMatch;
struct envoy_api_v2_listener_FilterChain;
struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate;
struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet;
struct envoy_api_v2_listener_ListenerFilter;
typedef struct envoy_api_v2_listener_Filter envoy_api_v2_listener_Filter;
typedef struct envoy_api_v2_listener_FilterChainMatch envoy_api_v2_listener_FilterChainMatch;
typedef struct envoy_api_v2_listener_FilterChain envoy_api_v2_listener_FilterChain;
typedef struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate envoy_api_v2_listener_ListenerFilterChainMatchPredicate;
typedef struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet;
typedef struct envoy_api_v2_listener_ListenerFilter envoy_api_v2_listener_ListenerFilter;
extern const upb_msglayout envoy_api_v2_listener_Filter_msginit;
extern const upb_msglayout envoy_api_v2_listener_FilterChainMatch_msginit;
extern const upb_msglayout envoy_api_v2_listener_FilterChain_msginit;
extern const upb_msglayout envoy_api_v2_listener_ListenerFilterChainMatchPredicate_msginit;
extern const upb_msglayout envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_msginit;
extern const upb_msglayout envoy_api_v2_listener_ListenerFilter_msginit;
struct envoy_api_v2_auth_DownstreamTlsContext;
struct envoy_api_v2_core_CidrRange;
struct envoy_api_v2_core_Metadata;
struct envoy_api_v2_core_TransportSocket;
struct envoy_type_Int32Range;
struct google_protobuf_Any;
struct google_protobuf_BoolValue;
struct google_protobuf_Struct;
struct google_protobuf_UInt32Value;
extern const upb_msglayout envoy_api_v2_auth_DownstreamTlsContext_msginit;
extern const upb_msglayout envoy_api_v2_core_CidrRange_msginit;
extern const upb_msglayout envoy_api_v2_core_Metadata_msginit;
extern const upb_msglayout envoy_api_v2_core_TransportSocket_msginit;
extern const upb_msglayout envoy_type_Int32Range_msginit;
extern const upb_msglayout google_protobuf_Any_msginit;
extern const upb_msglayout google_protobuf_BoolValue_msginit;
extern const upb_msglayout google_protobuf_Struct_msginit;
extern const upb_msglayout google_protobuf_UInt32Value_msginit;

typedef enum {
  envoy_api_v2_listener_FilterChainMatch_ANY = 0,
  envoy_api_v2_listener_FilterChainMatch_LOCAL = 1,
  envoy_api_v2_listener_FilterChainMatch_EXTERNAL = 2
} envoy_api_v2_listener_FilterChainMatch_ConnectionSourceType;


/* envoy.api.v2.listener.Filter */

UPB_INLINE envoy_api_v2_listener_Filter *envoy_api_v2_listener_Filter_new(upb_arena *arena) {
  return (envoy_api_v2_listener_Filter *)upb_msg_new(&envoy_api_v2_listener_Filter_msginit, arena);
}
UPB_INLINE envoy_api_v2_listener_Filter *envoy_api_v2_listener_Filter_parse(const char *buf, size_t size,
                        upb_arena *arena) {
  envoy_api_v2_listener_Filter *ret = envoy_api_v2_listener_Filter_new(arena);
  return (ret && upb_decode(buf, size, ret, &envoy_api_v2_listener_Filter_msginit, arena)) ? ret : NULL;
}
UPB_INLINE char *envoy_api_v2_listener_Filter_serialize(const envoy_api_v2_listener_Filter *msg, upb_arena *arena, size_t *len) {
  return upb_encode(msg, &envoy_api_v2_listener_Filter_msginit, arena, len);
}

typedef enum {
  envoy_api_v2_listener_Filter_config_type_config = 2,
  envoy_api_v2_listener_Filter_config_type_typed_config = 4,
  envoy_api_v2_listener_Filter_config_type_NOT_SET = 0
} envoy_api_v2_listener_Filter_config_type_oneofcases;
UPB_INLINE envoy_api_v2_listener_Filter_config_type_oneofcases envoy_api_v2_listener_Filter_config_type_case(const envoy_api_v2_listener_Filter* msg) { return (envoy_api_v2_listener_Filter_config_type_oneofcases)UPB_FIELD_AT(msg, int32_t, UPB_SIZE(12, 24)); }

UPB_INLINE upb_strview envoy_api_v2_listener_Filter_name(const envoy_api_v2_listener_Filter *msg) { return UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(0, 0)); }
UPB_INLINE bool envoy_api_v2_listener_Filter_has_config(const envoy_api_v2_listener_Filter *msg) { return _upb_has_oneof_field(msg, UPB_SIZE(12, 24), 2); }
UPB_INLINE const struct google_protobuf_Struct* envoy_api_v2_listener_Filter_config(const envoy_api_v2_listener_Filter *msg) { return UPB_READ_ONEOF(msg, const struct google_protobuf_Struct*, UPB_SIZE(8, 16), UPB_SIZE(12, 24), 2, NULL); }
UPB_INLINE bool envoy_api_v2_listener_Filter_has_typed_config(const envoy_api_v2_listener_Filter *msg) { return _upb_has_oneof_field(msg, UPB_SIZE(12, 24), 4); }
UPB_INLINE const struct google_protobuf_Any* envoy_api_v2_listener_Filter_typed_config(const envoy_api_v2_listener_Filter *msg) { return UPB_READ_ONEOF(msg, const struct google_protobuf_Any*, UPB_SIZE(8, 16), UPB_SIZE(12, 24), 4, NULL); }

UPB_INLINE void envoy_api_v2_listener_Filter_set_name(envoy_api_v2_listener_Filter *msg, upb_strview value) {
  UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(0, 0)) = value;
}
UPB_INLINE void envoy_api_v2_listener_Filter_set_config(envoy_api_v2_listener_Filter *msg, struct google_protobuf_Struct* value) {
  UPB_WRITE_ONEOF(msg, struct google_protobuf_Struct*, UPB_SIZE(8, 16), value, UPB_SIZE(12, 24), 2);
}
UPB_INLINE struct google_protobuf_Struct* envoy_api_v2_listener_Filter_mutable_config(envoy_api_v2_listener_Filter *msg, upb_arena *arena) {
  struct google_protobuf_Struct* sub = (struct google_protobuf_Struct*)envoy_api_v2_listener_Filter_config(msg);
  if (sub == NULL) {
    sub = (struct google_protobuf_Struct*)upb_msg_new(&google_protobuf_Struct_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_Filter_set_config(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_Filter_set_typed_config(envoy_api_v2_listener_Filter *msg, struct google_protobuf_Any* value) {
  UPB_WRITE_ONEOF(msg, struct google_protobuf_Any*, UPB_SIZE(8, 16), value, UPB_SIZE(12, 24), 4);
}
UPB_INLINE struct google_protobuf_Any* envoy_api_v2_listener_Filter_mutable_typed_config(envoy_api_v2_listener_Filter *msg, upb_arena *arena) {
  struct google_protobuf_Any* sub = (struct google_protobuf_Any*)envoy_api_v2_listener_Filter_typed_config(msg);
  if (sub == NULL) {
    sub = (struct google_protobuf_Any*)upb_msg_new(&google_protobuf_Any_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_Filter_set_typed_config(msg, sub);
  }
  return sub;
}

/* envoy.api.v2.listener.FilterChainMatch */

UPB_INLINE envoy_api_v2_listener_FilterChainMatch *envoy_api_v2_listener_FilterChainMatch_new(upb_arena *arena) {
  return (envoy_api_v2_listener_FilterChainMatch *)upb_msg_new(&envoy_api_v2_listener_FilterChainMatch_msginit, arena);
}
UPB_INLINE envoy_api_v2_listener_FilterChainMatch *envoy_api_v2_listener_FilterChainMatch_parse(const char *buf, size_t size,
                        upb_arena *arena) {
  envoy_api_v2_listener_FilterChainMatch *ret = envoy_api_v2_listener_FilterChainMatch_new(arena);
  return (ret && upb_decode(buf, size, ret, &envoy_api_v2_listener_FilterChainMatch_msginit, arena)) ? ret : NULL;
}
UPB_INLINE char *envoy_api_v2_listener_FilterChainMatch_serialize(const envoy_api_v2_listener_FilterChainMatch *msg, upb_arena *arena, size_t *len) {
  return upb_encode(msg, &envoy_api_v2_listener_FilterChainMatch_msginit, arena, len);
}

UPB_INLINE const struct envoy_api_v2_core_CidrRange* const* envoy_api_v2_listener_FilterChainMatch_prefix_ranges(const envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) { return (const struct envoy_api_v2_core_CidrRange* const*)_upb_array_accessor(msg, UPB_SIZE(32, 56), len); }
UPB_INLINE upb_strview envoy_api_v2_listener_FilterChainMatch_address_suffix(const envoy_api_v2_listener_FilterChainMatch *msg) { return UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(8, 8)); }
UPB_INLINE const struct google_protobuf_UInt32Value* envoy_api_v2_listener_FilterChainMatch_suffix_len(const envoy_api_v2_listener_FilterChainMatch *msg) { return UPB_FIELD_AT(msg, const struct google_protobuf_UInt32Value*, UPB_SIZE(24, 40)); }
UPB_INLINE const struct envoy_api_v2_core_CidrRange* const* envoy_api_v2_listener_FilterChainMatch_source_prefix_ranges(const envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) { return (const struct envoy_api_v2_core_CidrRange* const*)_upb_array_accessor(msg, UPB_SIZE(36, 64), len); }
UPB_INLINE uint32_t const* envoy_api_v2_listener_FilterChainMatch_source_ports(const envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) { return (uint32_t const*)_upb_array_accessor(msg, UPB_SIZE(40, 72), len); }
UPB_INLINE const struct google_protobuf_UInt32Value* envoy_api_v2_listener_FilterChainMatch_destination_port(const envoy_api_v2_listener_FilterChainMatch *msg) { return UPB_FIELD_AT(msg, const struct google_protobuf_UInt32Value*, UPB_SIZE(28, 48)); }
UPB_INLINE upb_strview envoy_api_v2_listener_FilterChainMatch_transport_protocol(const envoy_api_v2_listener_FilterChainMatch *msg) { return UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(16, 24)); }
UPB_INLINE upb_strview const* envoy_api_v2_listener_FilterChainMatch_application_protocols(const envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) { return (upb_strview const*)_upb_array_accessor(msg, UPB_SIZE(44, 80), len); }
UPB_INLINE upb_strview const* envoy_api_v2_listener_FilterChainMatch_server_names(const envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) { return (upb_strview const*)_upb_array_accessor(msg, UPB_SIZE(48, 88), len); }
UPB_INLINE int32_t envoy_api_v2_listener_FilterChainMatch_source_type(const envoy_api_v2_listener_FilterChainMatch *msg) { return UPB_FIELD_AT(msg, int32_t, UPB_SIZE(0, 0)); }

UPB_INLINE struct envoy_api_v2_core_CidrRange** envoy_api_v2_listener_FilterChainMatch_mutable_prefix_ranges(envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) {
  return (struct envoy_api_v2_core_CidrRange**)_upb_array_mutable_accessor(msg, UPB_SIZE(32, 56), len);
}
UPB_INLINE struct envoy_api_v2_core_CidrRange** envoy_api_v2_listener_FilterChainMatch_resize_prefix_ranges(envoy_api_v2_listener_FilterChainMatch *msg, size_t len, upb_arena *arena) {
  return (struct envoy_api_v2_core_CidrRange**)_upb_array_resize_accessor(msg, UPB_SIZE(32, 56), len, UPB_SIZE(4, 8), UPB_TYPE_MESSAGE, arena);
}
UPB_INLINE struct envoy_api_v2_core_CidrRange* envoy_api_v2_listener_FilterChainMatch_add_prefix_ranges(envoy_api_v2_listener_FilterChainMatch *msg, upb_arena *arena) {
  struct envoy_api_v2_core_CidrRange* sub = (struct envoy_api_v2_core_CidrRange*)upb_msg_new(&envoy_api_v2_core_CidrRange_msginit, arena);
  bool ok = _upb_array_append_accessor(
      msg, UPB_SIZE(32, 56), UPB_SIZE(4, 8), UPB_TYPE_MESSAGE, &sub, arena);
  if (!ok) return NULL;
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_FilterChainMatch_set_address_suffix(envoy_api_v2_listener_FilterChainMatch *msg, upb_strview value) {
  UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(8, 8)) = value;
}
UPB_INLINE void envoy_api_v2_listener_FilterChainMatch_set_suffix_len(envoy_api_v2_listener_FilterChainMatch *msg, struct google_protobuf_UInt32Value* value) {
  UPB_FIELD_AT(msg, struct google_protobuf_UInt32Value*, UPB_SIZE(24, 40)) = value;
}
UPB_INLINE struct google_protobuf_UInt32Value* envoy_api_v2_listener_FilterChainMatch_mutable_suffix_len(envoy_api_v2_listener_FilterChainMatch *msg, upb_arena *arena) {
  struct google_protobuf_UInt32Value* sub = (struct google_protobuf_UInt32Value*)envoy_api_v2_listener_FilterChainMatch_suffix_len(msg);
  if (sub == NULL) {
    sub = (struct google_protobuf_UInt32Value*)upb_msg_new(&google_protobuf_UInt32Value_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_FilterChainMatch_set_suffix_len(msg, sub);
  }
  return sub;
}
UPB_INLINE struct envoy_api_v2_core_CidrRange** envoy_api_v2_listener_FilterChainMatch_mutable_source_prefix_ranges(envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) {
  return (struct envoy_api_v2_core_CidrRange**)_upb_array_mutable_accessor(msg, UPB_SIZE(36, 64), len);
}
UPB_INLINE struct envoy_api_v2_core_CidrRange** envoy_api_v2_listener_FilterChainMatch_resize_source_prefix_ranges(envoy_api_v2_listener_FilterChainMatch *msg, size_t len, upb_arena *arena) {
  return (struct envoy_api_v2_core_CidrRange**)_upb_array_resize_accessor(msg, UPB_SIZE(36, 64), len, UPB_SIZE(4, 8), UPB_TYPE_MESSAGE, arena);
}
UPB_INLINE struct envoy_api_v2_core_CidrRange* envoy_api_v2_listener_FilterChainMatch_add_source_prefix_ranges(envoy_api_v2_listener_FilterChainMatch *msg, upb_arena *arena) {
  struct envoy_api_v2_core_CidrRange* sub = (struct envoy_api_v2_core_CidrRange*)upb_msg_new(&envoy_api_v2_core_CidrRange_msginit, arena);
  bool ok = _upb_array_append_accessor(
      msg, UPB_SIZE(36, 64), UPB_SIZE(4, 8), UPB_TYPE_MESSAGE, &sub, arena);
  if (!ok) return NULL;
  return sub;
}
UPB_INLINE uint32_t* envoy_api_v2_listener_FilterChainMatch_mutable_source_ports(envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) {
  return (uint32_t*)_upb_array_mutable_accessor(msg, UPB_SIZE(40, 72), len);
}
UPB_INLINE uint32_t* envoy_api_v2_listener_FilterChainMatch_resize_source_ports(envoy_api_v2_listener_FilterChainMatch *msg, size_t len, upb_arena *arena) {
  return (uint32_t*)_upb_array_resize_accessor(msg, UPB_SIZE(40, 72), len, UPB_SIZE(4, 4), UPB_TYPE_UINT32, arena);
}
UPB_INLINE bool envoy_api_v2_listener_FilterChainMatch_add_source_ports(envoy_api_v2_listener_FilterChainMatch *msg, uint32_t val, upb_arena *arena) {
  return _upb_array_append_accessor(
      msg, UPB_SIZE(40, 72), UPB_SIZE(4, 4), UPB_TYPE_UINT32, &val, arena);
}
UPB_INLINE void envoy_api_v2_listener_FilterChainMatch_set_destination_port(envoy_api_v2_listener_FilterChainMatch *msg, struct google_protobuf_UInt32Value* value) {
  UPB_FIELD_AT(msg, struct google_protobuf_UInt32Value*, UPB_SIZE(28, 48)) = value;
}
UPB_INLINE struct google_protobuf_UInt32Value* envoy_api_v2_listener_FilterChainMatch_mutable_destination_port(envoy_api_v2_listener_FilterChainMatch *msg, upb_arena *arena) {
  struct google_protobuf_UInt32Value* sub = (struct google_protobuf_UInt32Value*)envoy_api_v2_listener_FilterChainMatch_destination_port(msg);
  if (sub == NULL) {
    sub = (struct google_protobuf_UInt32Value*)upb_msg_new(&google_protobuf_UInt32Value_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_FilterChainMatch_set_destination_port(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_FilterChainMatch_set_transport_protocol(envoy_api_v2_listener_FilterChainMatch *msg, upb_strview value) {
  UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(16, 24)) = value;
}
UPB_INLINE upb_strview* envoy_api_v2_listener_FilterChainMatch_mutable_application_protocols(envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) {
  return (upb_strview*)_upb_array_mutable_accessor(msg, UPB_SIZE(44, 80), len);
}
UPB_INLINE upb_strview* envoy_api_v2_listener_FilterChainMatch_resize_application_protocols(envoy_api_v2_listener_FilterChainMatch *msg, size_t len, upb_arena *arena) {
  return (upb_strview*)_upb_array_resize_accessor(msg, UPB_SIZE(44, 80), len, UPB_SIZE(8, 16), UPB_TYPE_STRING, arena);
}
UPB_INLINE bool envoy_api_v2_listener_FilterChainMatch_add_application_protocols(envoy_api_v2_listener_FilterChainMatch *msg, upb_strview val, upb_arena *arena) {
  return _upb_array_append_accessor(
      msg, UPB_SIZE(44, 80), UPB_SIZE(8, 16), UPB_TYPE_STRING, &val, arena);
}
UPB_INLINE upb_strview* envoy_api_v2_listener_FilterChainMatch_mutable_server_names(envoy_api_v2_listener_FilterChainMatch *msg, size_t *len) {
  return (upb_strview*)_upb_array_mutable_accessor(msg, UPB_SIZE(48, 88), len);
}
UPB_INLINE upb_strview* envoy_api_v2_listener_FilterChainMatch_resize_server_names(envoy_api_v2_listener_FilterChainMatch *msg, size_t len, upb_arena *arena) {
  return (upb_strview*)_upb_array_resize_accessor(msg, UPB_SIZE(48, 88), len, UPB_SIZE(8, 16), UPB_TYPE_STRING, arena);
}
UPB_INLINE bool envoy_api_v2_listener_FilterChainMatch_add_server_names(envoy_api_v2_listener_FilterChainMatch *msg, upb_strview val, upb_arena *arena) {
  return _upb_array_append_accessor(
      msg, UPB_SIZE(48, 88), UPB_SIZE(8, 16), UPB_TYPE_STRING, &val, arena);
}
UPB_INLINE void envoy_api_v2_listener_FilterChainMatch_set_source_type(envoy_api_v2_listener_FilterChainMatch *msg, int32_t value) {
  UPB_FIELD_AT(msg, int32_t, UPB_SIZE(0, 0)) = value;
}

/* envoy.api.v2.listener.FilterChain */

UPB_INLINE envoy_api_v2_listener_FilterChain *envoy_api_v2_listener_FilterChain_new(upb_arena *arena) {
  return (envoy_api_v2_listener_FilterChain *)upb_msg_new(&envoy_api_v2_listener_FilterChain_msginit, arena);
}
UPB_INLINE envoy_api_v2_listener_FilterChain *envoy_api_v2_listener_FilterChain_parse(const char *buf, size_t size,
                        upb_arena *arena) {
  envoy_api_v2_listener_FilterChain *ret = envoy_api_v2_listener_FilterChain_new(arena);
  return (ret && upb_decode(buf, size, ret, &envoy_api_v2_listener_FilterChain_msginit, arena)) ? ret : NULL;
}
UPB_INLINE char *envoy_api_v2_listener_FilterChain_serialize(const envoy_api_v2_listener_FilterChain *msg, upb_arena *arena, size_t *len) {
  return upb_encode(msg, &envoy_api_v2_listener_FilterChain_msginit, arena, len);
}

UPB_INLINE const envoy_api_v2_listener_FilterChainMatch* envoy_api_v2_listener_FilterChain_filter_chain_match(const envoy_api_v2_listener_FilterChain *msg) { return UPB_FIELD_AT(msg, const envoy_api_v2_listener_FilterChainMatch*, UPB_SIZE(8, 16)); }
UPB_INLINE const struct envoy_api_v2_auth_DownstreamTlsContext* envoy_api_v2_listener_FilterChain_tls_context(const envoy_api_v2_listener_FilterChain *msg) { return UPB_FIELD_AT(msg, const struct envoy_api_v2_auth_DownstreamTlsContext*, UPB_SIZE(12, 24)); }
UPB_INLINE const envoy_api_v2_listener_Filter* const* envoy_api_v2_listener_FilterChain_filters(const envoy_api_v2_listener_FilterChain *msg, size_t *len) { return (const envoy_api_v2_listener_Filter* const*)_upb_array_accessor(msg, UPB_SIZE(28, 56), len); }
UPB_INLINE const struct google_protobuf_BoolValue* envoy_api_v2_listener_FilterChain_use_proxy_proto(const envoy_api_v2_listener_FilterChain *msg) { return UPB_FIELD_AT(msg, const struct google_protobuf_BoolValue*, UPB_SIZE(16, 32)); }
UPB_INLINE const struct envoy_api_v2_core_Metadata* envoy_api_v2_listener_FilterChain_metadata(const envoy_api_v2_listener_FilterChain *msg) { return UPB_FIELD_AT(msg, const struct envoy_api_v2_core_Metadata*, UPB_SIZE(20, 40)); }
UPB_INLINE const struct envoy_api_v2_core_TransportSocket* envoy_api_v2_listener_FilterChain_transport_socket(const envoy_api_v2_listener_FilterChain *msg) { return UPB_FIELD_AT(msg, const struct envoy_api_v2_core_TransportSocket*, UPB_SIZE(24, 48)); }
UPB_INLINE upb_strview envoy_api_v2_listener_FilterChain_name(const envoy_api_v2_listener_FilterChain *msg) { return UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(0, 0)); }

UPB_INLINE void envoy_api_v2_listener_FilterChain_set_filter_chain_match(envoy_api_v2_listener_FilterChain *msg, envoy_api_v2_listener_FilterChainMatch* value) {
  UPB_FIELD_AT(msg, envoy_api_v2_listener_FilterChainMatch*, UPB_SIZE(8, 16)) = value;
}
UPB_INLINE struct envoy_api_v2_listener_FilterChainMatch* envoy_api_v2_listener_FilterChain_mutable_filter_chain_match(envoy_api_v2_listener_FilterChain *msg, upb_arena *arena) {
  struct envoy_api_v2_listener_FilterChainMatch* sub = (struct envoy_api_v2_listener_FilterChainMatch*)envoy_api_v2_listener_FilterChain_filter_chain_match(msg);
  if (sub == NULL) {
    sub = (struct envoy_api_v2_listener_FilterChainMatch*)upb_msg_new(&envoy_api_v2_listener_FilterChainMatch_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_FilterChain_set_filter_chain_match(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_FilterChain_set_tls_context(envoy_api_v2_listener_FilterChain *msg, struct envoy_api_v2_auth_DownstreamTlsContext* value) {
  UPB_FIELD_AT(msg, struct envoy_api_v2_auth_DownstreamTlsContext*, UPB_SIZE(12, 24)) = value;
}
UPB_INLINE struct envoy_api_v2_auth_DownstreamTlsContext* envoy_api_v2_listener_FilterChain_mutable_tls_context(envoy_api_v2_listener_FilterChain *msg, upb_arena *arena) {
  struct envoy_api_v2_auth_DownstreamTlsContext* sub = (struct envoy_api_v2_auth_DownstreamTlsContext*)envoy_api_v2_listener_FilterChain_tls_context(msg);
  if (sub == NULL) {
    sub = (struct envoy_api_v2_auth_DownstreamTlsContext*)upb_msg_new(&envoy_api_v2_auth_DownstreamTlsContext_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_FilterChain_set_tls_context(msg, sub);
  }
  return sub;
}
UPB_INLINE envoy_api_v2_listener_Filter** envoy_api_v2_listener_FilterChain_mutable_filters(envoy_api_v2_listener_FilterChain *msg, size_t *len) {
  return (envoy_api_v2_listener_Filter**)_upb_array_mutable_accessor(msg, UPB_SIZE(28, 56), len);
}
UPB_INLINE envoy_api_v2_listener_Filter** envoy_api_v2_listener_FilterChain_resize_filters(envoy_api_v2_listener_FilterChain *msg, size_t len, upb_arena *arena) {
  return (envoy_api_v2_listener_Filter**)_upb_array_resize_accessor(msg, UPB_SIZE(28, 56), len, UPB_SIZE(4, 8), UPB_TYPE_MESSAGE, arena);
}
UPB_INLINE struct envoy_api_v2_listener_Filter* envoy_api_v2_listener_FilterChain_add_filters(envoy_api_v2_listener_FilterChain *msg, upb_arena *arena) {
  struct envoy_api_v2_listener_Filter* sub = (struct envoy_api_v2_listener_Filter*)upb_msg_new(&envoy_api_v2_listener_Filter_msginit, arena);
  bool ok = _upb_array_append_accessor(
      msg, UPB_SIZE(28, 56), UPB_SIZE(4, 8), UPB_TYPE_MESSAGE, &sub, arena);
  if (!ok) return NULL;
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_FilterChain_set_use_proxy_proto(envoy_api_v2_listener_FilterChain *msg, struct google_protobuf_BoolValue* value) {
  UPB_FIELD_AT(msg, struct google_protobuf_BoolValue*, UPB_SIZE(16, 32)) = value;
}
UPB_INLINE struct google_protobuf_BoolValue* envoy_api_v2_listener_FilterChain_mutable_use_proxy_proto(envoy_api_v2_listener_FilterChain *msg, upb_arena *arena) {
  struct google_protobuf_BoolValue* sub = (struct google_protobuf_BoolValue*)envoy_api_v2_listener_FilterChain_use_proxy_proto(msg);
  if (sub == NULL) {
    sub = (struct google_protobuf_BoolValue*)upb_msg_new(&google_protobuf_BoolValue_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_FilterChain_set_use_proxy_proto(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_FilterChain_set_metadata(envoy_api_v2_listener_FilterChain *msg, struct envoy_api_v2_core_Metadata* value) {
  UPB_FIELD_AT(msg, struct envoy_api_v2_core_Metadata*, UPB_SIZE(20, 40)) = value;
}
UPB_INLINE struct envoy_api_v2_core_Metadata* envoy_api_v2_listener_FilterChain_mutable_metadata(envoy_api_v2_listener_FilterChain *msg, upb_arena *arena) {
  struct envoy_api_v2_core_Metadata* sub = (struct envoy_api_v2_core_Metadata*)envoy_api_v2_listener_FilterChain_metadata(msg);
  if (sub == NULL) {
    sub = (struct envoy_api_v2_core_Metadata*)upb_msg_new(&envoy_api_v2_core_Metadata_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_FilterChain_set_metadata(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_FilterChain_set_transport_socket(envoy_api_v2_listener_FilterChain *msg, struct envoy_api_v2_core_TransportSocket* value) {
  UPB_FIELD_AT(msg, struct envoy_api_v2_core_TransportSocket*, UPB_SIZE(24, 48)) = value;
}
UPB_INLINE struct envoy_api_v2_core_TransportSocket* envoy_api_v2_listener_FilterChain_mutable_transport_socket(envoy_api_v2_listener_FilterChain *msg, upb_arena *arena) {
  struct envoy_api_v2_core_TransportSocket* sub = (struct envoy_api_v2_core_TransportSocket*)envoy_api_v2_listener_FilterChain_transport_socket(msg);
  if (sub == NULL) {
    sub = (struct envoy_api_v2_core_TransportSocket*)upb_msg_new(&envoy_api_v2_core_TransportSocket_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_FilterChain_set_transport_socket(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_FilterChain_set_name(envoy_api_v2_listener_FilterChain *msg, upb_strview value) {
  UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(0, 0)) = value;
}

/* envoy.api.v2.listener.ListenerFilterChainMatchPredicate */

UPB_INLINE envoy_api_v2_listener_ListenerFilterChainMatchPredicate *envoy_api_v2_listener_ListenerFilterChainMatchPredicate_new(upb_arena *arena) {
  return (envoy_api_v2_listener_ListenerFilterChainMatchPredicate *)upb_msg_new(&envoy_api_v2_listener_ListenerFilterChainMatchPredicate_msginit, arena);
}
UPB_INLINE envoy_api_v2_listener_ListenerFilterChainMatchPredicate *envoy_api_v2_listener_ListenerFilterChainMatchPredicate_parse(const char *buf, size_t size,
                        upb_arena *arena) {
  envoy_api_v2_listener_ListenerFilterChainMatchPredicate *ret = envoy_api_v2_listener_ListenerFilterChainMatchPredicate_new(arena);
  return (ret && upb_decode(buf, size, ret, &envoy_api_v2_listener_ListenerFilterChainMatchPredicate_msginit, arena)) ? ret : NULL;
}
UPB_INLINE char *envoy_api_v2_listener_ListenerFilterChainMatchPredicate_serialize(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, upb_arena *arena, size_t *len) {
  return upb_encode(msg, &envoy_api_v2_listener_ListenerFilterChainMatchPredicate_msginit, arena, len);
}

typedef enum {
  envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_or_match = 1,
  envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_and_match = 2,
  envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_not_match = 3,
  envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_any_match = 4,
  envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_destination_port_range = 5,
  envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_NOT_SET = 0
} envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_oneofcases;
UPB_INLINE envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_oneofcases envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_case(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate* msg) { return (envoy_api_v2_listener_ListenerFilterChainMatchPredicate_rule_oneofcases)UPB_FIELD_AT(msg, int32_t, UPB_SIZE(4, 8)); }

UPB_INLINE bool envoy_api_v2_listener_ListenerFilterChainMatchPredicate_has_or_match(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return _upb_has_oneof_field(msg, UPB_SIZE(4, 8), 1); }
UPB_INLINE const envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_or_match(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return UPB_READ_ONEOF(msg, const envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet*, UPB_SIZE(0, 0), UPB_SIZE(4, 8), 1, NULL); }
UPB_INLINE bool envoy_api_v2_listener_ListenerFilterChainMatchPredicate_has_and_match(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return _upb_has_oneof_field(msg, UPB_SIZE(4, 8), 2); }
UPB_INLINE const envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_and_match(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return UPB_READ_ONEOF(msg, const envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet*, UPB_SIZE(0, 0), UPB_SIZE(4, 8), 2, NULL); }
UPB_INLINE bool envoy_api_v2_listener_ListenerFilterChainMatchPredicate_has_not_match(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return _upb_has_oneof_field(msg, UPB_SIZE(4, 8), 3); }
UPB_INLINE const envoy_api_v2_listener_ListenerFilterChainMatchPredicate* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_not_match(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return UPB_READ_ONEOF(msg, const envoy_api_v2_listener_ListenerFilterChainMatchPredicate*, UPB_SIZE(0, 0), UPB_SIZE(4, 8), 3, NULL); }
UPB_INLINE bool envoy_api_v2_listener_ListenerFilterChainMatchPredicate_has_any_match(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return _upb_has_oneof_field(msg, UPB_SIZE(4, 8), 4); }
UPB_INLINE bool envoy_api_v2_listener_ListenerFilterChainMatchPredicate_any_match(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return UPB_READ_ONEOF(msg, bool, UPB_SIZE(0, 0), UPB_SIZE(4, 8), 4, false); }
UPB_INLINE bool envoy_api_v2_listener_ListenerFilterChainMatchPredicate_has_destination_port_range(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return _upb_has_oneof_field(msg, UPB_SIZE(4, 8), 5); }
UPB_INLINE const struct envoy_type_Int32Range* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_destination_port_range(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg) { return UPB_READ_ONEOF(msg, const struct envoy_type_Int32Range*, UPB_SIZE(0, 0), UPB_SIZE(4, 8), 5, NULL); }

UPB_INLINE void envoy_api_v2_listener_ListenerFilterChainMatchPredicate_set_or_match(envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet* value) {
  UPB_WRITE_ONEOF(msg, envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet*, UPB_SIZE(0, 0), value, UPB_SIZE(4, 8), 1);
}
UPB_INLINE struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_mutable_or_match(envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, upb_arena *arena) {
  struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet* sub = (struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet*)envoy_api_v2_listener_ListenerFilterChainMatchPredicate_or_match(msg);
  if (sub == NULL) {
    sub = (struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet*)upb_msg_new(&envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_ListenerFilterChainMatchPredicate_set_or_match(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_ListenerFilterChainMatchPredicate_set_and_match(envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet* value) {
  UPB_WRITE_ONEOF(msg, envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet*, UPB_SIZE(0, 0), value, UPB_SIZE(4, 8), 2);
}
UPB_INLINE struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_mutable_and_match(envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, upb_arena *arena) {
  struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet* sub = (struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet*)envoy_api_v2_listener_ListenerFilterChainMatchPredicate_and_match(msg);
  if (sub == NULL) {
    sub = (struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet*)upb_msg_new(&envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_ListenerFilterChainMatchPredicate_set_and_match(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_ListenerFilterChainMatchPredicate_set_not_match(envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, envoy_api_v2_listener_ListenerFilterChainMatchPredicate* value) {
  UPB_WRITE_ONEOF(msg, envoy_api_v2_listener_ListenerFilterChainMatchPredicate*, UPB_SIZE(0, 0), value, UPB_SIZE(4, 8), 3);
}
UPB_INLINE struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_mutable_not_match(envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, upb_arena *arena) {
  struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate* sub = (struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate*)envoy_api_v2_listener_ListenerFilterChainMatchPredicate_not_match(msg);
  if (sub == NULL) {
    sub = (struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate*)upb_msg_new(&envoy_api_v2_listener_ListenerFilterChainMatchPredicate_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_ListenerFilterChainMatchPredicate_set_not_match(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_ListenerFilterChainMatchPredicate_set_any_match(envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, bool value) {
  UPB_WRITE_ONEOF(msg, bool, UPB_SIZE(0, 0), value, UPB_SIZE(4, 8), 4);
}
UPB_INLINE void envoy_api_v2_listener_ListenerFilterChainMatchPredicate_set_destination_port_range(envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, struct envoy_type_Int32Range* value) {
  UPB_WRITE_ONEOF(msg, struct envoy_type_Int32Range*, UPB_SIZE(0, 0), value, UPB_SIZE(4, 8), 5);
}
UPB_INLINE struct envoy_type_Int32Range* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_mutable_destination_port_range(envoy_api_v2_listener_ListenerFilterChainMatchPredicate *msg, upb_arena *arena) {
  struct envoy_type_Int32Range* sub = (struct envoy_type_Int32Range*)envoy_api_v2_listener_ListenerFilterChainMatchPredicate_destination_port_range(msg);
  if (sub == NULL) {
    sub = (struct envoy_type_Int32Range*)upb_msg_new(&envoy_type_Int32Range_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_ListenerFilterChainMatchPredicate_set_destination_port_range(msg, sub);
  }
  return sub;
}

/* envoy.api.v2.listener.ListenerFilterChainMatchPredicate.MatchSet */

UPB_INLINE envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet *envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_new(upb_arena *arena) {
  return (envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet *)upb_msg_new(&envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_msginit, arena);
}
UPB_INLINE envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet *envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_parse(const char *buf, size_t size,
                        upb_arena *arena) {
  envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet *ret = envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_new(arena);
  return (ret && upb_decode(buf, size, ret, &envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_msginit, arena)) ? ret : NULL;
}
UPB_INLINE char *envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_serialize(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet *msg, upb_arena *arena, size_t *len) {
  return upb_encode(msg, &envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_msginit, arena, len);
}

UPB_INLINE const envoy_api_v2_listener_ListenerFilterChainMatchPredicate* const* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_rules(const envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet *msg, size_t *len) { return (const envoy_api_v2_listener_ListenerFilterChainMatchPredicate* const*)_upb_array_accessor(msg, UPB_SIZE(0, 0), len); }

UPB_INLINE envoy_api_v2_listener_ListenerFilterChainMatchPredicate** envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_mutable_rules(envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet *msg, size_t *len) {
  return (envoy_api_v2_listener_ListenerFilterChainMatchPredicate**)_upb_array_mutable_accessor(msg, UPB_SIZE(0, 0), len);
}
UPB_INLINE envoy_api_v2_listener_ListenerFilterChainMatchPredicate** envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_resize_rules(envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet *msg, size_t len, upb_arena *arena) {
  return (envoy_api_v2_listener_ListenerFilterChainMatchPredicate**)_upb_array_resize_accessor(msg, UPB_SIZE(0, 0), len, UPB_SIZE(4, 8), UPB_TYPE_MESSAGE, arena);
}
UPB_INLINE struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate* envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet_add_rules(envoy_api_v2_listener_ListenerFilterChainMatchPredicate_MatchSet *msg, upb_arena *arena) {
  struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate* sub = (struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate*)upb_msg_new(&envoy_api_v2_listener_ListenerFilterChainMatchPredicate_msginit, arena);
  bool ok = _upb_array_append_accessor(
      msg, UPB_SIZE(0, 0), UPB_SIZE(4, 8), UPB_TYPE_MESSAGE, &sub, arena);
  if (!ok) return NULL;
  return sub;
}

/* envoy.api.v2.listener.ListenerFilter */

UPB_INLINE envoy_api_v2_listener_ListenerFilter *envoy_api_v2_listener_ListenerFilter_new(upb_arena *arena) {
  return (envoy_api_v2_listener_ListenerFilter *)upb_msg_new(&envoy_api_v2_listener_ListenerFilter_msginit, arena);
}
UPB_INLINE envoy_api_v2_listener_ListenerFilter *envoy_api_v2_listener_ListenerFilter_parse(const char *buf, size_t size,
                        upb_arena *arena) {
  envoy_api_v2_listener_ListenerFilter *ret = envoy_api_v2_listener_ListenerFilter_new(arena);
  return (ret && upb_decode(buf, size, ret, &envoy_api_v2_listener_ListenerFilter_msginit, arena)) ? ret : NULL;
}
UPB_INLINE char *envoy_api_v2_listener_ListenerFilter_serialize(const envoy_api_v2_listener_ListenerFilter *msg, upb_arena *arena, size_t *len) {
  return upb_encode(msg, &envoy_api_v2_listener_ListenerFilter_msginit, arena, len);
}

typedef enum {
  envoy_api_v2_listener_ListenerFilter_config_type_config = 2,
  envoy_api_v2_listener_ListenerFilter_config_type_typed_config = 3,
  envoy_api_v2_listener_ListenerFilter_config_type_NOT_SET = 0
} envoy_api_v2_listener_ListenerFilter_config_type_oneofcases;
UPB_INLINE envoy_api_v2_listener_ListenerFilter_config_type_oneofcases envoy_api_v2_listener_ListenerFilter_config_type_case(const envoy_api_v2_listener_ListenerFilter* msg) { return (envoy_api_v2_listener_ListenerFilter_config_type_oneofcases)UPB_FIELD_AT(msg, int32_t, UPB_SIZE(16, 32)); }

UPB_INLINE upb_strview envoy_api_v2_listener_ListenerFilter_name(const envoy_api_v2_listener_ListenerFilter *msg) { return UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(0, 0)); }
UPB_INLINE bool envoy_api_v2_listener_ListenerFilter_has_config(const envoy_api_v2_listener_ListenerFilter *msg) { return _upb_has_oneof_field(msg, UPB_SIZE(16, 32), 2); }
UPB_INLINE const struct google_protobuf_Struct* envoy_api_v2_listener_ListenerFilter_config(const envoy_api_v2_listener_ListenerFilter *msg) { return UPB_READ_ONEOF(msg, const struct google_protobuf_Struct*, UPB_SIZE(12, 24), UPB_SIZE(16, 32), 2, NULL); }
UPB_INLINE bool envoy_api_v2_listener_ListenerFilter_has_typed_config(const envoy_api_v2_listener_ListenerFilter *msg) { return _upb_has_oneof_field(msg, UPB_SIZE(16, 32), 3); }
UPB_INLINE const struct google_protobuf_Any* envoy_api_v2_listener_ListenerFilter_typed_config(const envoy_api_v2_listener_ListenerFilter *msg) { return UPB_READ_ONEOF(msg, const struct google_protobuf_Any*, UPB_SIZE(12, 24), UPB_SIZE(16, 32), 3, NULL); }
UPB_INLINE const envoy_api_v2_listener_ListenerFilterChainMatchPredicate* envoy_api_v2_listener_ListenerFilter_filter_disabled(const envoy_api_v2_listener_ListenerFilter *msg) { return UPB_FIELD_AT(msg, const envoy_api_v2_listener_ListenerFilterChainMatchPredicate*, UPB_SIZE(8, 16)); }

UPB_INLINE void envoy_api_v2_listener_ListenerFilter_set_name(envoy_api_v2_listener_ListenerFilter *msg, upb_strview value) {
  UPB_FIELD_AT(msg, upb_strview, UPB_SIZE(0, 0)) = value;
}
UPB_INLINE void envoy_api_v2_listener_ListenerFilter_set_config(envoy_api_v2_listener_ListenerFilter *msg, struct google_protobuf_Struct* value) {
  UPB_WRITE_ONEOF(msg, struct google_protobuf_Struct*, UPB_SIZE(12, 24), value, UPB_SIZE(16, 32), 2);
}
UPB_INLINE struct google_protobuf_Struct* envoy_api_v2_listener_ListenerFilter_mutable_config(envoy_api_v2_listener_ListenerFilter *msg, upb_arena *arena) {
  struct google_protobuf_Struct* sub = (struct google_protobuf_Struct*)envoy_api_v2_listener_ListenerFilter_config(msg);
  if (sub == NULL) {
    sub = (struct google_protobuf_Struct*)upb_msg_new(&google_protobuf_Struct_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_ListenerFilter_set_config(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_ListenerFilter_set_typed_config(envoy_api_v2_listener_ListenerFilter *msg, struct google_protobuf_Any* value) {
  UPB_WRITE_ONEOF(msg, struct google_protobuf_Any*, UPB_SIZE(12, 24), value, UPB_SIZE(16, 32), 3);
}
UPB_INLINE struct google_protobuf_Any* envoy_api_v2_listener_ListenerFilter_mutable_typed_config(envoy_api_v2_listener_ListenerFilter *msg, upb_arena *arena) {
  struct google_protobuf_Any* sub = (struct google_protobuf_Any*)envoy_api_v2_listener_ListenerFilter_typed_config(msg);
  if (sub == NULL) {
    sub = (struct google_protobuf_Any*)upb_msg_new(&google_protobuf_Any_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_ListenerFilter_set_typed_config(msg, sub);
  }
  return sub;
}
UPB_INLINE void envoy_api_v2_listener_ListenerFilter_set_filter_disabled(envoy_api_v2_listener_ListenerFilter *msg, envoy_api_v2_listener_ListenerFilterChainMatchPredicate* value) {
  UPB_FIELD_AT(msg, envoy_api_v2_listener_ListenerFilterChainMatchPredicate*, UPB_SIZE(8, 16)) = value;
}
UPB_INLINE struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate* envoy_api_v2_listener_ListenerFilter_mutable_filter_disabled(envoy_api_v2_listener_ListenerFilter *msg, upb_arena *arena) {
  struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate* sub = (struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate*)envoy_api_v2_listener_ListenerFilter_filter_disabled(msg);
  if (sub == NULL) {
    sub = (struct envoy_api_v2_listener_ListenerFilterChainMatchPredicate*)upb_msg_new(&envoy_api_v2_listener_ListenerFilterChainMatchPredicate_msginit, arena);
    if (!sub) return NULL;
    envoy_api_v2_listener_ListenerFilter_set_filter_disabled(msg, sub);
  }
  return sub;
}

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port_undef.inc"

#endif  /* ENVOY_API_V2_LISTENER_LISTENER_COMPONENTS_PROTO_UPB_H_ */
