#region Copyright notice and license

// Copyright 2015 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#endregion

using System;
using System.Collections.Generic;

namespace Grpc.Core.Logging
{
    /// <summary>Standard logging levels.</summary>
    public enum LogLevel
    {
        /// <summary>
        /// Debug severity.
        /// </summary>
        Debug = 0,
        /// <summary>
        /// Info severity.
        /// </summary>
        Info,
        /// <summary>
        /// Warning severity.
        /// </summary>
        Warning,
        /// <summary>
        /// Error severity.
        /// </summary>
        Error,
        /// <summary>
        /// Logging is off.
        /// </summary>
        Off = int.MaxValue
    }
}
